<?php 
// WU-forecast.php script by Ken True - webmaster@saratoga-weather.org
//
// Version 1.00 - 02-Sep-2007 - initial release
// Version 1.01 - 09-Sep-2007 - added selective printing and translation for 'Chance of precipitation'
// Version 1.02 - 18-Sep-2007 - fixed typo on line 533 ($PrintMode instead of $printMode)
// Version 1.03 - 31-Jan-2008 - modified to support new Wunderground page layout and icon set
// Version 1.04 - 21-Feb-2008 - modified to support minor WU website change and added support for
//                              Carterlake/AJAX/PHP template integration
// Version 1.05 - 30-Mar-2008 - added units selector C/F/both function for icon legend
// Version 1.06 - 21-Apr-2008 - added $SITE['xlateCOP'] for World-Multi-lingual support
// Version 1.07 - 22-Apr-2008 - added iconv() to convert UTF-8 to targeted character set $SITE['charset']
// Version 1.08 - 01-Oct-2008 - added support for WU display of Tropical data when it appears
// Version 1.09 - 02-Dec-2008 - fixes for WeatherUnderground website changes
// Version 1.10 - 15-Dec-2008 - fixes for F/C temperature for WU website changes
// Version 1.11 - 19-Dec-2008 - fixes for WU website changes
// Version 1.12 - 22-Dec-2008 - fix to remove '\r\n2000\r\n' from WU website returned HTML
// Version 1.13 - 26-Jan-2011 - added support for global $cacheFileDir for cache files
// Version 1.14 - 03-Feb-2011 - fixes for WeatherUnderground major website changes
// Version 1.15 - 07-Feb-2011 - more fixes for multilingual usage
// Version 1.16 - 06-Mar-2011 - added foldIconRow option to display icons in two-rows of 5
// Version 1.17 - 30-May-2011 - added fix for premature halt if missing data from WU (heat index/wind chill)
// Version 1.18 - 07-Sep-2011 - fixes for WeatherUnderground website changes
// Version 1.19 - 09-Sep-2011 - added $maxForecasts to limit text forecasts to display (7 days worth)
// Version 1.20 - 13-Sep-2011 - added new method for short-forecast from icon if forecast text too long
// Version 1.21 - 14-Sep-2011 - added fix to correct invalid WU HTML in text forecast area
// Version 1.22 - 16-Sep-2011 - fixes for WeatherUnderground website changes
// Version 1.23 - 01-Oct-2011 - added support for alternative animated icon set from http://www.meteotreviglio.com/
// Version 1.24 - 25-Oct-2011 - fixes for WeatherUnderground website changes
// Version 1.25 - 05-Nov-2011 - fixes to remove Notice: errata
// Version 1.26 - 17-Nov-2011 - add lang=he support (UTF-8) and multi-forecast capability
// Version 1.27 - 31-Dec-2011 - remove 'Weather Forecast ' from $WUforecastcity
// Version 1.28 - 03-Feb-2012 - added diagnostics if access blocked by WU website
// Version 1.29 - 16-Feb-2012 - fixes for WeatherUnderground website changes
// Version 1.30 - 01-Jun-2012 - fixes for WeatherUnderground website changes (title and temperature)
//
$Version = "WU-forecast.php (ML) Version 1.30- 01-Jun-2012";
//
// error_reporting(E_ALL);  // uncomment to turn on full error reporting
//
// script available at http://saratoga-weather.org/scripts.php
//  
// you may copy/modify/use this script as you see fit,
// no warranty is expressed or implied.
//
// This script parses the WeatherUnderground forecast html page and loads icons/text into
//  arrays so you can use them in your weather website.  
//
// NOTE: You must leave an attribution link to weatherunderground.com in the output page.
//
//  The script DOES NOT work for locations in the USA, US territories or Canada. There are
//  other scripts for NOAA and EC forecasts for you to use instead of this script.
//
// output: creates XHTML 1.0-Strict HTML page (or inclusion)
//
// by default, the script uses the WeatherUnderground icons shown at
//    http://www.wunderground.com/graphics/conds/ for the conditions display.
//
// This script may optionally use the NOAA icons instead of linking to the
// WeatherUnderground Icons .. if you use this option, then the NOAA icons need to be placed in the path 
// where the original NOAA icons are located: \forecast\images\
// (so make a folder in your web HTML root called "forecast", then make a folder in it 
// called "images", and place the icons in this folder)
// The icon set is available at http://saratoga-weather.org/carterlake-icons.zip 
// If you have the icon set from:
//   http://members.cox.net/carterlakeweather/forecasticons.zip (380K)
// then please download http://saratoga-weather.org/carterlake-icons-addon.zip and
// upload to your /forecast/images directory.  
//
//
// Options on URL:
//
//   inc=Y            - omit <HTML><HEAD></HEAD><BODY> and </BODY></HTML> from output
//   heading=n        - (default)='y' suppress printing of heading (forecast city/by/date)
//   icons=n          - (default)='y' suppress printing of the icons+conditions+temp+wind+UV
//   text=n           - (default)='y' suppress printing of the periods/forecast text
//
//
//  You can also invoke these options directly in the PHP like this
//
//    $doIncludeWU = true;
//    include("WU-forecast.php");  for just the text
//  or ------------
//    $doPrintWU = false;
//    include("WU-forecast.php");  for setting up the $WUforecast... variables without printing
//
//  or ------------
//    $doIncludeWU = true;
//    $doPrintHeadingWU = true;
//    $doPrintIconsWU = true;
//    $doPrintTextWU = false
//    include("WU-forecast.php");  include mode, print only heading and icon set
//
// Variables returned (useful for printing an icon or forecast or two...)
//
// $WUforecastcity 		- Name of city from WU Forecast header
//
// The following variables exist for $i=0 to $i= number of forecast periods minus 1
//  a loop of for ($i=0;$i<count($WUforecastday);$i++) { ... } will loop over the available 
//  values.
//
// $WUforecastday[$i]	- period of forecast
// $WUforecasttext[$i]	- text of forecast 
// $WUforecasttemp[$i]	- Temperature with text and formatting
// $WUforecastpop[$i]	- Number - Probabability of Precipitation ('',10,20, ... ,100)
// $WUforecasticon[$i]   - base name of icon graphic to use
// $WUforecastcond[$i]   - Short legend for forecast icon 
// $WUforecasticons[$i]  - Full icon with Period, <img> and Short legend.
//
// Settings ---------------------------------------------------------------
$iconDir ='./forecast/images/';           // directory for carterlake icons './forecast/images/'
$iconType = '.jpg';        // default type='.jpg' -- use '.gif' for animated icons from http://www.meteotreviglio.com/
//$iconDir ='';                           // set to '' to use the Wunderground icons instead
//
// view the forecast you want on www.wunderground.com using the language setting you desire,
// then copy the FULL url to the $WU_URL setting below.  Some examples are commented out below
//
// NOTE: don't use locations in the United States, US territories or Canada -- this script won't
//  process the Wunderground versions for those locations.
//
//$WU_URL = 'http://www.wunderground.com/cgi-bin/findweather/getForecast?query=95070'; // doesn't work
//$WU_URL = 'http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Calgary%2C+Canada'; // doesn't work
//$WU_URL = 'http://www.wunderground.com/cgi-bin/findweather/getForecast?query=San+Juan%2C+Puerto+Rico'; // doesn't work
//
//$WU_URL = 'http://www.wunderground.com/global/stations/93119.html'; // Awhitu, Waiuku New Zealand
$WU_URL = 'http://www.wunderground.com/global/stations/06075.html';
//$WU_URL = 'http://dutch.wunderground.com/global/stations/06370.html';
//$WU_URL = 'http://dutch.wunderground.com/cgi-bin/findweather/getForecast?query=Amsterdam%2C+Netherlands';
//$WU_URL = 'http://french.wunderground.com/cgi-bin/findweather/getForecast?query=Paris%2C+France';
//$WU_URL = 'http://swedish.wunderground.com/cgi-bin/findweather/getForecast?query=Stockholm%2C+Sweden';
//$WU_URL = 'http://norwegian.wunderground.com/cgi-bin/findweather/getForecast?query=Oslo%2C+Norway';
//$WU_URL = 'http://russian.wunderground.com/cgi-bin/findweather/getForecast?query=Moscow%2C+Russia';
//$WU_URL =  'http://greek.wunderground.com/cgi-bin/findweather/getForecast?query=Athens%2C+Greece';
//$WU_URL =  'http://hebrew.wunderground.com/cgi-bin/findweather/getForecast?query=Tel+Aviv%2C+Israel';
//$WU_URL = 'http://espanol.wunderground.com/cgi-bin/findweather/getForecast?query=Madrid%2C+Spain';
//$WU_URL = 'http://finnish.wunderground.com/cgi-bin/findweather/getForecast?query=Helsinki%2C+Finland';
//$WU_URL = 'http://deutsch.wunderground.com/global/stations/10389.html';
//$WU_URL = 'http://portuguese.wunderground.com/cgi-bin/findweather/getForecast?query=Lisbon%2C+Portugal';
//$WU_URL = 'http://www.wunderground.com/cgi-bin/findweather/getForecast?query=London%2C+United+Kingdom';
//$WU_URL = 'http://www.wunderground.com/global/stations/03628.html';
//
// The optional multi-city forecast .. make sure the first entry is for the $WU_URL location
// The contents will be replaced by $SITE['WUforecasts'] if specified in your Settings.php
/*
$WUforecasts = array(
 // Location|forecast-URL  (separated by | characters)
'Auckland|http://www.wunderground.com/global/stations/93119.html', // Awhitu, Waiuku New Zealand
'Amsterdam|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Amsterdam%2C+Netherlands',
'Paris|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Paris%2C+France',
'Stockholm|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Stockholm%2C+Sweden',
'Oslo|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Oslo%2C+Norway',
'Moscow|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Moscow%2C+Russia',
'Athens|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Athens%2C+Greece',
'Tel Aviv|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Tel+Aviv%2C+Israel',
'Madrid|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Madrid%2C+Spain',
'Helsinki|http://www.wunderground.com/cgi-bin/findweather/getForecast?query=Helsinki%2C+Finland',
 
); 
//*/

//
$maxWidth = '640px';                      // max width of tables (could be '100%')
$maxIcons = 10;                           // max number of icons to display
$maxForecasts = 14;                       // max number of Text forecast periods to display
$maxForecastLegendWords = 4;              // more words in forecast legend than this number will use our forecast words 
$numIconsInFoldedRow = 5;                 // if words cause overflow of $maxWidth pixels, then put this num of icons in rows
$autoSetTemplate = true;                  // =true set icons based on wide/narrow template design
$cacheFileDir = './';                     // default cache file directory
$cacheName = "WU-forecast.txt";           // locally cached page from WU
$refetchSeconds = 1800;                    // cache lifetime (1800sec = 30 minutes)
$xlateCOP = 'Chance of precipitation';     // change to local language if needed like
//$xlateCOP = 'Kans op neerslag';   //  Dutch example
$showTempsAs  = 'C';                  // under icons, 'C'=Centigrade only, 'F'=Fahrenheit only, 'B' = both
$charsetOutput = 'ISO-8859-1';        // default character encoding of output
$lang = 'en';	// default language
$foldIconRow = true;  // display icons in rows of 5 if long texts are found
$RTLlang = ',he,jp,cn,';  // languages that use right-to-left order
// ---- end of settings ---------------------------------------------------
// overrides from Settings.php if available
global $SITE;
if (isset($SITE['WUforecasts']))   {$WUforecasts = $SITE['WUforecasts']; }
if (isset($SITE['fcsturlWU'])) 	{$WU_URL = $SITE['fcsturlWU'];}
if (isset($SITE['fcsticonsdir'])) 	{$iconDir = $SITE['fcsticonsdir'];}
if (isset($SITE['fcsticonstype'])) 	{$iconType = $SITE['fcsticonstype'];}
if (isset($SITE['uomTemp']))	{$uomTemp = $SITE['uomTemp'];}
if (isset($SITE['xlateCOP']))	{$xlateCOP = $SITE['xlateCOP'];}
if (isset($LANGLOOKUP['Chance of precipitation'])) {
  $xlateCOP = $LANGLOOKUP['Chance of precipitation'];
}
if (isset($SITE['charset']))	{$charsetOutput = strtoupper($SITE['charset']); }
if (isset($SITE['lang']))		{$lang = $SITE['lang'];}
if(isset($SITE['cacheFileDir']))     {$cacheFileDir = $SITE['cacheFileDir']; }
if(isset($SITE['foldIconRow']))     {$foldIconRow = $SITE['foldIconRow']; }
if(isset($SITE['RTL-LANG']))     {$RTLlang = $SITE['RTL-LANG']; }
// end of overrides from Settings.php
//
// -------------------begin code ------------------------------------------
if (isset($_REQUEST['sce']) && strtolower($_REQUEST['sce']) == 'view' ) {
   //--self downloader --
   $filenameReal = __FILE__;
   $download_size = filesize($filenameReal);
   header('Pragma: public');
   header('Cache-Control: private');
   header('Cache-Control: no-cache, must-revalidate');
   header("Content-type: text/plain");
   header("Accept-Ranges: bytes");
   header("Content-Length: $download_size");
   header('Connection: close');
   
   readfile($filenameReal);
   exit;
}
$Status = "<!-- $Version -->\n";
//------------------------------------------------

$NWSiconlist = array(
 // WU Icon name => NWS icon name // WU meaning
 'chanceflurries.gif' 	=> array('sn.jpg','Chance flurries'), // Chance of Flurries
 'chancerain.gif' 		=> array('hi_shwrs.jpg','Chance rain'), // Chance of Rain
 'chancesleet.gif' 		=> array('ip.jpg','Chance sleet'), // Chance of Sleet
 'chancesnow.gif' 		=> array('sn.jpg','Chance snow'), // Chance of Snow
 'chancetstorms.gif' 	=> array('hi_tsra.jpg','Chance thunderstorms'), // Chance of Thunderstorms
 'clear.gif' 			=> array('skc.jpg','Clear'), // Clear
 'cloudy.gif' 			=> array('ovc.jpg','Cloudy'), // Cloudy
 'flurries.gif' 		=> array('sn.jpg','Flurries'), // Flurries
 'fog.gif' 				=> array('fg.jpg','Fog'), // Fog
 'hazy.gif' 			=> array('fg.jpg','Hazy'), // Hazy
 'mostlycloudy.gif' 	=> array('bkn.jpg','Mostly cloudy'), // Mostly Cloudy
 'mostlysunny.gif' 		=> array('sct.jpg','Partly cloudy'), // Mostly Sunny
 'partlycloudy.gif' 	=> array('sct.jpg','Partly cloudy'), // Partly Cloudy
 'partlysunny.gif' 		=> array('bkn.jpg','Mostly sunny'), // Partly Sunny
 'rain.gif' 			=> array('ra.jpg','Rain'), // Rain 
 'sleet.gif' 			=> array('ip.jpg','Sleet'), // Sleet
 'sleat.gif' 			=> array('ip.jpg','Sleet'), // Sleet (other spelling)
 'snow.gif' 			=> array('sn.jpg','Snow'), // Snow
 'sunny.gif' 			=> array('skc.jpg','Sunny'), // Sunny
 'tstorms.gif' 			=> array('tsra.jpg','Thunderstorms'), // Thunderstorms
 'unknown.gif' 			=> array('na.jpg',''), // Unknown
 'nt_chanceflurries.gif' => array('nsn.jpg','Chance flurries'), // Chance of Flurries
 'nt_chancerain.gif' 	=> array('hi_nshwrs.jpg','Chance rain'), // Chance of Rain
 'nt_chancesleet.gif' 	=> array('ip.jpg','Chance sleet'), // Chance of Sleet
 'nt_chancesnow.gif' 	=> array('nsn.jpg','Chance snow'), // Chance of Snow
 'nt_chancetstorms.gif' => array('hi_ntsra.jpg','Chance thunderstorms'), // Chance of Thunderstorms
 'nt_clear.gif' 		=> array('nskc.jpg','Clear'), // Clear
 'nt_cloudy.gif' 		=> array('novc.jpg','Cloudy'), // Cloudy
 'nt_flurries.gif' 		=> array('nsn.jpg','Flurries'), // Flurries
 'nt_fog.gif' 			=> array('nfg.jpg','Fog'), // Fog
 'nt_hazy.gif' 			=> array('nfg.jpg','Hazy'), // Hazy
 'nt_mostlycloudy.gif' 	=> array('nbkn.jpg','Mostly cloudy'), // Mostly Cloudy
 'nt_mostlysunny.gif' 	=> array('nsct.jpg','Partly cloudy'), // Mostly Sunny
 'nt_partlycloudy.gif' 	=> array('nsct.jpg','Partly cloudy'), // Partly Cloudy
 'nt_partlysunny.gif' 	=> array('nbkn.jpg','Mostly cloudy'), // Partly Sunny
 'nt_rain.gif' 			=> array('nra.jpg','Rain'), // Rain 
 'nt_sleet.gif' 		=> array('ip.jpg','Sleet'), // Sleet
 'nt_sleat.gif'			=> array('ip.jpg','Sleet'), // sleet (other spelling)
 'nt_snow.gif' 			=> array('nsn.jpg','Snow'), // Snow
 'nt_sunny.gif' 		=> array('nskc.jpg','Sunny'), // Sunny
 'nt_tstorms.gif' 		=> array('ntsra.jpg','Thunderstorms'), // Thunderstorms
 'nt_unknown.gif' 		=> array('na.jpg',''), // Unknown
 '.gif' 		        => array('na.jpg',''), // Unknown
) ;
//
/* translation entries needed:

langlookup|Chance flurries|Chance flurries|
langlookup|Chance rain|Chance rain|
langlookup|Chance sleet|Chance sleet|
langlookup|Chance snow|Chance snow|
langlookup|Chance thunderstorms|Chance thunderstorms|
langlookup|Clear|Clear|
langlookup|Cloudy|Cloudy|
langlookup|Flurries|Flurries|
langlookup|Fog|Fog|
langlookup|Hazy|Hazy|
langlookup|Mostly cloudy|Mostly cloudy|
langlookup|Partly cloudy|Partly cloudy|
langlookup|Mostly sunny|Mostly sunny|
langlookup|Rain|Rain|
langlookup|Sleet|Sleet|
langlookup|Snow|Snow|
langlookup|Sunny|Sunny|
langlookup|Thunderstorms|Thunderstorms|

*/
if(!function_exists('langtransstr')) {
	// shim function if not running in template set
	function langtransstr($input) { return($input); }
}
$doRTL = (strpos($RTLlang,$lang) !== false)?true:false;  // format RTL language in Right-to-left in output

// get the selected forecast location code
$haveIndex = '0';
if (!empty($_GET['z']) && preg_match("/^[0-9]+$/i", htmlspecialchars($_GET['z']))) {
  $haveIndex = htmlspecialchars(strip_tags($_GET['z']));  // valid zone syntax from input
} 

if(!isset($WUforecasts[0])) {
	// print "<!-- making NWSforecasts array default -->\n";
	$WUforecasts = array("|$WU_URL"); // create default entry
}
//  print "<!-- NWSforecasts\n".print_r($WUforecasts,true). " -->\n";
// Set the default zone. The first entry in the $SITE['NWSforecasts'] array.
list($Nl,$Nn) = explode('|',$WUforecasts[0].'|||');
$FCSTlocation = $Nl;
$WU_URL = $Nn;

if(!isset($WUforecasts[$haveIndex])) {
	$haveIndex = 0;
}

// locations added to the drop down menu and set selected zone values
$dDownMenu = '';
for ($m=0;$m<count($WUforecasts);$m++) { // for each locations
  list($Nlocation,$Nname) = explode('|',$WUforecasts[$m].'|||');
  $seltext = '';
  if($haveIndex == $m) {
    $FCSTlocation = $Nlocation;
    $WU_URL = $Nname;
	$seltext = ' selected="selected" ';
  }
  $dDownMenu .= "     <option value=\"$m\"$seltext>".langtransstr($Nlocation)."</option>\n";
}

// build the drop down menu
$ddMenu = '';
// create menu if at least two locations are listed in the array
if (isset($WUforecasts[0]) and isset($WUforecasts[1])) {
	if($doRTL) {$RTLopt = ' style="direction: rtl;"'; } else {$RTLopt = '';}; 
	$ddMenu .= '<tr align="center">
      <td style="font-size: 14px; font-family: Arial, Helvetica, sans-serif">
      <script type="text/javascript">
        <!--
        function menu_goto( menuform ){
         selecteditem = menuform.logfile.selectedIndex ;
         logfile = menuform.logfile.options[ selecteditem ].value ;
         if (logfile.length != 0) {
          location.href = logfile ;
         }
        }
        //-->
      </script>
     <form action="" method="get">
     <p><select name="z" onchange="this.form.submit()"'.$RTLopt.'>
     <option value=""> - '.langtransstr('Select Forecast').' - </option>
' . $dDownMenu .
		$ddMenu . '     </select></p>
     <div><noscript><pre><input name="submit" type="submit" value="'.langtransstr('Get Forecast').'" /></pre></noscript></div>
     </form>
    </td>
   </tr>
';
}

$Force = false;

if (isset($_REQUEST['force']) and  $_REQUEST['force']=="1" ) {
  $Force = true;
}

$doDebug = false;
if (isset($_REQUEST['debug']) and strtolower($_REQUEST['debug'])=='y' ) {
  $doDebug = true;
}

$fileName = $WU_URL;
if ($doDebug) {
  $Status .= "<!-- WU URL: $fileName -->\n";
}

if (isset($uomTemp) and $showTempsAs <> 'B') { // use Settings.php Temp units only
   $showTempAs = preg_match('|C|i',$uomTemp) ? 'C' : 'F';
   $Status .= "<!-- temps in $showTempAs -->\n";
}

if ($autoSetTemplate and isset($_SESSION['CSSwidescreen'])) {
	if($_SESSION['CSSwidescreen'] == true) {
	   $maxWidth = '900px';
	   $maxIcons = 14;
	   $maxForecasts = 14;
	   $numIconsInFoldedRow = 7;
	   $Status .= "<!-- autoSetTemplate using ".$SITE['CSSwideOrNarrowDefault']." aspect. -->\n";	
	}
	if($_SESSION['CSSwidescreen'] == false) {
	   $maxWidth = '640px';
	   $maxIcons = 10;
	   $maxForecasts = 14;
	   $numIconsInFoldedRow = 5;
	   $Status .= "<!-- autoSetTemplate using ".$SITE['CSSwideOrNarrowDefault']." aspect. -->\n";	
	}
}

$cacheName = $cacheFileDir . $cacheName;
$cacheName = preg_replace('|\.txt|is',"-$haveIndex-$lang.txt",$cacheName); // unique cache per language used

if (isset($SITE['WULanguages'][$lang])) {
    $Status .= "<!-- WU URL is '".$fileName."' -->\n";
     $fileName = preg_replace('|http://([^\.]+)\.(.*)$|is',
	    "http://".$SITE['WULanguages'][$lang].".\\2",
		$fileName);
	$Status .= "<!-- WU URL set to '".$fileName."' -->\n";
  
}

// The number 1800 below is the number of seconds the cache will be used instead of pulling a new file
// 1800 = 60s x 30m so it retreives every 30 minutes. 

if (! $Force and file_exists($cacheName) and filemtime($cacheName) + $refetchSeconds > time()) {
      $html = implode('', file($cacheName)); 
      $Status .= "<!-- loading from $cacheName (" . strlen($html) . " bytes) -->\n"; 
  } else { 
      $Status .= "<!-- loading from $fileName. -->\n"; 
      $html = fetchUrlWithoutHangingWU($fileName,$cacheName); 
	  
    $RC = '';
	if (preg_match("|^HTTP\/\S+ (.*)\r\n|",$html,$matches)) {
	    $RC = trim($matches[1]);
	}
	$Status .= "<!-- RC=$RC, bytes=" . strlen($html) . " -->\n";
	if (preg_match('|30\d |',$RC)) { // handle possible blocked redirect
	   preg_match('|Location: (\S+)|is',$html,$matches);
	   if(isset($matches[1])) {
		  $sURL = $matches[1];
		  if(preg_match('|opendns.com|i',$sURL)) {
			  $Status .= "<!--  NOT following to $sURL --->\n";
		  } else {
			$Status .= "<!-- following to $sURL --->\n";
		
			$html = fetchUrlWithoutHangingWU($sURL,false);
			$RC = '';
			if (preg_match("|^HTTP\/\S+ (.*)\r\n|",$html,$matches)) {
				$RC = trim($matches[1]);
			}
			$Status .= "<!-- RC=$RC, bytes=" . strlen($html) . " -->\n";
		  }
	   }
    }
      $fp = fopen($cacheName, "w"); 
	  if (!$fp) { 
	    $Status .= "<!-- unable to open $cacheName for writing. -->\n"; 
	  } else {
        $write = fputs($fp, $html); 
        fclose($fp);  
		$Status .= "<!-- saved cache to $cacheName (". strlen($html) . " bytes) -->\n";
	  } 
} 


  preg_match('|charset="{0,1}(\S+)"{0,1}|i',$html,$matches);
  
  if (isset($matches[1])) {
    $charsetInput = strtoupper($matches[1]);
  } else {
    $charsetInput = 'UTF-8';
  }
  
 $doIconv = ($charsetInput == $charsetOutput)?false:true; // only do iconv() if sets are different
 
 $Status .= "<!-- using charsetInput='$charsetInput' charsetOutput='$charsetOutput' doIconv='$doIconv' doRTL='$doRTL' -->\n";
 //  process the file .. select out the 7-day forecast part of the page
  $UnSupported = false;
//  preg_match_all('|<div class="fctZoneHeader">(.*)</table>\s+</div>\s+</div>\s+</div>\s+<div id="conds_details_radarbox">|Uis',$html,$matches); // this pulls the text forecast chunk
//  preg_match_all('|<div id="fct_day_\d+" class="fctDay .*>(.*)</div>\s+<div class="clear"></div>|Uis',$html,$matches); // select details day-only 
  preg_match_all('|<div class="fLeft w295 p10">(.*)</table>|Uis',$html,$matches); // select day/night text forecasts+icon 

   if($doDebug) {
     $Status .= "<!-- first slice for table matches\n" . print_r($matches[1],true) . "  end first slice matches-->\n";	
   }
  if(count($matches[1]) < 1 and strlen($html) > 1000) { // oops... not parsable.. give 'em the news later
//  $Status .= "<!-- matches\n" . print_r($matches,true) . "-->\n";
  $UnSupported = true;	  
  }
 
 
 $rows = $matches[1];
 
/* rows now contains

    [0] => 
		<div class="fctDayDate">Wednesday, 7</div>
		<div class="fctDayContent">
		<div class="fctCondIcon"><a href="" class="iconSwitchMed"><img src="http://icons-ak.wxug.com/i/c/k/chancerain.gif" alt="Chance of Rain" title="/mnt/s3/wundFct//IM/IMIDTJYL18.hdf.gz" class="condIcon" /></a></div>
		<div class="fctHiLow">
		<span class="b">17</span> | 10 &deg;C
		</div>
		<div class="fctDayConditions">
		Chance of Rain
		</div>
		<div class="fctDayPop">
		<div class="popText">Chance of<br />Precipitation</div>
		<div class="popValue">50%
    [1] => 
		<div class="fctDayDate">Thursday, 8</div>
		<div class="fctDayContent">
		<div class="fctCondIcon"><a href="" class="iconSwitchMed"><img src="http://icons-ak.wxug.com/i/c/k/chancerain.gif" alt="Chance of Rain" title="/mnt/s3/wundFct//IM/IMIDTJYL18.hdf.gz" class="condIcon" /></a></div>
		<div class="fctHiLow">
		<span class="b">16</span> | 10 &deg;C
		</div>
		<div class="fctDayConditions">
		Chance of Rain
		</div>
		<div class="fctDayPop">
		<div class="popText">Chance of<br />Precipitation</div>
		<div class="popValue">20%


*/

/*
		<h3 id="fct_titlebar">
		<a href="/history/airport/EKVD/2012/6/1/MonthlyHistory.html#calendar" id="viewCal">View Calendar</a>
		7-Day Weather Forecast
		for
		<span class="b">Vamdrup</span>
		</h3>
*/
  preg_match('|<h3 id="fct_titlebar">(.*?)</h3>|is',$html,$matches); // look for forecast header
   if($doDebug) {
     $Status .= "<!-- forecast title matches\n" . print_r($matches,true) . "-->\n";
   }
   $temp = explode("\n",trim(strip_tags($matches[1])));
   $WUtitle = trim($temp[1]);
   if($doIconv) {$WUtitle = iconv($charsetInput,$charsetOutput.'//TRANSLIT',$WUtitle);}
   if($doDebug) {
     $Status .= "<!-- WUtitle='$WUtitle' -->\n";
   }
  
   preg_match_all('|<title>(.*?)</title>|is',$html,$matches); // look for city name in title
   if($doDebug) {
     $Status .= "<!-- city title\n" . print_r($matches,true) . "-->\n";
   }
   $temp = explode(",",$matches[1][0]); // split after city name
   $WUforecastcity = trim($temp[0]);
   $WUforecastcity = preg_replace('|Weather Forecast |i','',$WUforecastcity);
   if($doIconv) {$WUforecastcity = iconv($charsetInput,$charsetOutput.'//TRANSLIT',$WUforecastcity);}
   if($doDebug) {
     $Status .= "<!-- WUforecastcity='$WUforecastcity' -->\n";
   }
   $WUtitle = preg_replace("|$WUforecastcity|",'',$WUtitle);
   if($doDebug) {
     $Status .= "<!-- WUtitle='$WUtitle' -->\n";
   }
   

$n = 0; // counter for arrays to stuff data in




  for ($i=0;$i<count($rows);$i++) {  // 
 
/* <!-- table rows  this is all different
Array
(

    [0] => 
  <div class="b">Tuesday</div>
  <table cellspacing="5" cellpadding="0" class="full">
  <tr>
  <td class="vaT"><a href="" class="iconSwitchMed"><img src="http://icons-ak.wxug.com/i/c/k/chancerain.gif" alt="chancerain" class="condIcon" /></a></td>
  <td class="vaT">Mostly cloudy in the morning, then overcast with a chance of rain. High of 18. Winds from the SSW at 35 km/h. Chance of rain 40% with rainfall amounts near 3 mm.
</td>
  </tr>
  
    [1] => 
  <div class="b">Tuesday Night</div>
  <table cellspacing="5" cellpadding="0" class="full">
  <tr>
  <td class="vaT"><a href="" class="iconSwitchMed"><img src="http://icons-ak.wxug.com/i/c/k/nt_rain.gif" alt="nt_rain" class="condIcon" /></a></td>
  <td class="vaT">Overcast with rain. Low of 12. Winds from the SSW at 25-35 km/h. Chance of rain 80% with rainfall amounts near 9 mm.
</td>
  </tr>
	
*/   

   if($i >= $maxForecasts) {
	   break;  // finished with all useful ones
   }
   
 	if ( $doDebug) {
      $Status .= "<!-- processing rows[$i]='" . $rows[$i] . "' -->\n";
	}	
   

	preg_match('|src="([^"]+)"|i',$rows[$i],$temp); // extract raw image name
	$t = parse_url($temp[1]);
	$t = pathinfo($t['path']);
	$WUforecasticon[$n] = $t['basename'];
	if ($doDebug) {
      $Status .= "<!-- WUforecasticon[$n]='" . $WUforecasticon[$n] . "' -->\n";
	}	

    preg_match('|<div class="b">(.*?)</div>(.*)|is',$rows[$i],$temp); // extract period
//	  $Status .= "<!-- temp\n" . print_r($temp,true) . "-->\n";	 
	$WUforecastday[$n] = trim($temp[1]);
	if($doIconv) {$WUforecastday[$n] = iconv($charsetInput,$charsetOutput.'//TRANSLIT',$WUforecastday[$n]);}
	$WUforecasttitles[$n] = $WUforecastday[$n];
	if ($doDebug) {
      $Status .= "<!-- WUforecastday[$n]='" . $WUforecastday[$n] . "' -->\n";
	}	
    preg_match('|</td>\s+<td class="vaT">(.*)\s*</td>\s+</tr>|is',$rows[$i],$temp);
	$WUforecasttext[$n] = trim( preg_replace("|\n\s+|is",' ',$temp[1]) );
	if($doIconv) {$WUforecasttext[$n] = preg_replace('/(Chance of Precip.|Chance of precipitation:)/s',iconv($charsetOutput,$charsetInput,$xlateCOP).':',$WUforecasttext[$n]);}
	$WUforecasttext[$n] = trim(preg_replace('|<\?.*\?>|is','',$WUforecasttext[$n])); // remove <? END CHANCE OF PRECIP ? >
	if($doIconv) {$WUforecasttext[$n] = iconv($charsetInput,$charsetOutput.'//TRANSLIT',$WUforecasttext[$n]);}
	if ($doDebug) {
      $Status .= "<!-- WUforecasttext[$n]='" . $WUforecasttext[$n] . "' -->\n";
	}
	
	if (preg_match('|(\d+)\%|',$WUforecasttext[$n],$temp)) {
	  $WUforecastpop[$n] = $temp[1]; // got a PoP
	   } else {
	  $WUforecastpop[$n] = '';
	 }
	if ($doDebug) {
      $Status .= "<!-- WUforecastpop[$n]='" . $WUforecastpop[$n] . "' -->\n";
	}

/* samples of text forecasts:

    [0] => Mostly cloudy with a chance of rain in the morning, then partly cloudy with a chance of rain.
    [1] => High of 17.
    [2] => Winds from the WSW at 35 km/h.
    [3] => Chance of rain 50% with rainfall amounts near 1 mm.

    [0] => Overcast in the evening, then clear.
    [1] => Low of 10.
    [2] => Winds from the WNW at 20 km/h.

    [0] => Mostly cloudy in the morning, then overcast.
    [1] => High of 19 and humid.
    [2] => Winds from the South at 20 km/h.

    [0] => Overcast in the morning, then partly cloudy.
    [1] => Areas of fog early.
    [2] => High of 21.
    [3] => Winds from the South at 5 km/h.

    [0] => Partly cloudy in the evening, then mostly cloudy.
    [1] => Areas of fog overnight.
    [2] => Low of 12.
    [3] => Winds from the SSW at 15-25 km/h.

*/

// fixup temperature
    $doESfixup = false;
	preg_match_all('!\.\s+([^:]+)\:{0,1}\s+([\d|-]+)(&#176;|&deg;||C|F)!Uis',$WUforecasttext[$n],$temp);
	
	if(isset($temp[1][0]) and strpos($temp[1][0],'.') !== false) {
	   if( $doDebug) { $Status .= "<!-- prune temperature text from '".$temp[1][0]."'"; }	
	   $t = trim(substr($temp[1][0],strpos($temp[1][0],'.')+1));
	   $temp[1][0] = $t;	
	   if( $doDebug) { $Status .= " to '$t' -->\n"; }	
	}

	if(!isset($temp[2][0])) {
	   if( $doDebug) { $Status .= "<!-- failed first temperature match; trying again. -->\n"; }	
	   preg_match_all('!\.\s+([^:]+)\:\s+([\d|-]+)&#176;!s',$WUforecasttext[$n],$temp);
	}
	if(!isset($temp[2][0])) {
	   // . Maks.: 21&#176; C.
	   if( $doDebug) { $Status .= "<!-- failed second temperature match; trying again. -->\n"; }	
	   preg_match_all('!\.\s+(\S+)\s+\S+\s+([\d|-]+)&deg;!s',$WUforecasttext[$n],$temp);
	}
	if(!isset($temp[2][0])) {
	   // . Maks.: 21&#176; C.
	   if( $doDebug) { $Status .= "<!-- failed third temperature match; trying again. -->\n"; }
	   // this is a hack for the lang=es version which looks like
	   // Alta de nn
	   // for all of them High and Low.  Hopefully they'll add a degree sign and fix night 'Alta' to 'Min'	
	   preg_match_all('!\.\s+(Alta)\s+de\s+([\d|-]+)!s',$WUforecasttext[$n],$temp);
	   $doESfixup = true;
	}
	if($doRTL) { // try the Hebrew
	   if( $doDebug) { $Status .= "<!-- trying lang='he' temperature match. -->\n"; }
	   preg_match_all('! (\S+)\s{0,1}\xD7\x99\xD7\x9E\xD7\x95\xD7\x9D ([\d|-]+)!',$WUforecasttext[$n],$temp); // min/max for Hebrew
	}
	   
	if ( $doDebug) {
      $Status .= "<!-- temp[$n]='" . print_r($temp,true) . "' -->\n";
	}

    $color = '#FF0000';
	
	$tday = trim($WUforecastday[$n]);
	$yday = $tday;
	if($n > 0) {$yday = trim(strip_tags($WUforecastday[$n-1])); }
	$tdayblanks = count(explode(" ",$tday)) -1; // counts the number of blanks in todays name
	if($doDebug) {
	  $Status .= "<!-- DayNight tday'$tday' yday='$yday' tdayblanks=$tdayblanks -->\n";
	}

	if ( ($tdayblanks > 0) or 
	     (strlen($tday) > strlen($yday) and stripos($tday,$yday) !== false)
		  ) {
	  // the above IF checks to see if the todays name is longer than yesterdays name, and
	  // that yesterdays day name is present in todays name .. this probably means it's a
	  // night time forecast, so treat it as a 'low' temp, and change the icon to night-time
	  // if needed.
	  if($doDebug) {
		$Status .= "<!-- night time icon selected -->\n";
	  }
	  if(substr($WUforecasticon[$n],0,3) <> 'nt_') {
  	    $WUforecasticon[$n] = 'nt_'.$WUforecasticon[$n]; // funny.. they stopped using day/night icons
	  }
	  $color = '#0000FF';
	}
	// special handling for bad (currently) Spanish translation on WU
	if($doESfixup && isset($temp[1][0]) and $temp[1][0] == 'Alta') {
		$temp[1][0] = (substr($color,0,3) == "#FF")?'Mx':'Min';
	}
	// end special Spanish handling
	$WUforecasttemp[$n] = "<span style=\"color: $color;\">";
	$beforeError = error_reporting();
	error_reporting($beforeError & ! E_NOTICE);
	if($doIconv) {$thilo = iconv($charsetInput,$charsetOutput.'//TRANSLIT',$temp[1][0]); }
	  else { $thilo = $temp[1][0]; }
	error_reporting($beforeError);
	if ($doDebug) {
      $Status .= "<!-- $charsetInput temp[1][0]='" . $temp[1][0] . "' $charsetOutput thilo='$thilo' -->\n";
	}
	if(isset($temp[1][0]) and strlen($thilo) < 2) {$thilo = trim($temp[1][0]); } // in case the iconv fails
    if(isset($temp[2][0])) {$WUforecasttemp[$n] .= $thilo . ' '. preg_replace('| &deg;|','&deg;',$temp[2][0]);}
	$WUforecasttemp[$n] .= '</span>';  // assemble the temperature

	if ($doDebug) {
      $Status .= "<!-- forecasttemp[$n]='" . $WUforecasttemp[$n] . "' -->\n";
	}
	
	$temp = explode('.',$WUforecasttext[$n]); // split as sentences (sort of).
	
	$WUforecastcond[$n] = trim($temp[0]); // take first one as summary.
	if ($doDebug) {
      $Status .= "<!-- forecastcond[$n]='" . $WUforecastcond[$n] . "' -->\n";
	}
    if (count(explode(' ',$WUforecastcond[$n])) > $maxForecastLegendWords) {
	   if(function_exists('langtransstr')) {
         $WUforecastcond[$n] = langtransstr($NWSiconlist[$WUforecasticon[$n]][1]); // use our description instead
	   } else {
         $WUforecastcond[$n] = $NWSiconlist[$WUforecasticon[$n]][1]; // use our description instead
	   }
	   $Status .= "<!-- replaced forecastcond[$n]='" . $WUforecastcond[$n] . "' -->\n";
    }

	
	if(preg_match('| |',$WUforecastday[$n]) ) {
	  $WUforecastday[$n] = preg_replace('| |','<br/>',$WUforecastday[$n],1);
	} else {
	  $WUforecastday[$n] .= "<br/>";
	}
	
	$WUforecasticons[$n] = $WUforecastday[$n] . "<br/>" .
	     img_replace($WUforecasticon[$n],$WUforecastcond[$n],$WUforecastpop[$n]) . "<br/>" .
		 $WUforecastcond[$n];
		
	if ($doDebug) {
	  $Status .= "\n";
	}
	$n++;
}

// All finished with parsing, now prepare to print

	   $wdth = intval(100/count($WUforecasticons));
	   $ndays = intval(count($WUforecasticon)/2);
	   
	   $WUtitle = preg_replace('|5|i',$ndays,$WUtitle,1);
	
       $doNumIcons = $maxIcons;
	   if(count($WUforecasticons) < $maxIcons) { $doNumIcons = count($WUforecasticons); }

  $IncludeMode = false;
  $PrintMode = true;

  if (isset($doPrintWU) && ! $doPrintWU ) {
      print $Status;
      return;
  }
  if (isset($_REQUEST['inc']) && 
      strtolower($_REQUEST['inc']) == 'noprint' ) {
      print $Status;
	  return;
  }

if (isset($_REQUEST['inc']) && strtolower($_REQUEST['inc']) == 'y') {
  $IncludeMode = true;
}
if (isset($doIncludeWU)) {
  $IncludeMode = $doIncludeWU;
}

$printHeading = true;
$printIcons = true;
$printText = true;

if (isset($doPrintHeadingWU)) {
  $printHeading = $doPrintHeadingWU;
}
if (isset($_REQUEST['heading']) ) {
  $printHeading = substr(strtolower($_REQUEST['heading']),0,1) == 'y';
}

if (isset($doPrintIconsWU)) {
  $printIcons = $doPrintIconsWU;
}
if (isset($_REQUEST['icons']) ) {
  $printIcons = substr(strtolower($_REQUEST['icons']),0,1) == 'y';
}
if (isset($doPrintTextWU)) {
  $printText = $doPrintTextWU;
}
if (isset($_REQUEST['text']) ) {
  $printText = substr(strtolower($_REQUEST['text']),0,1) == 'y';
}


if (! $IncludeMode and $PrintMode) { ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>WeatherUnderground <?php echo $WUtitle . ' - ' . $WUforecastcity; ?></title>
    <meta http-equiv="Content-Type" content="text/html; charset=<?php echo $charsetOutput; ?>" />
</head>
<body style="font-family:Verdana, Arial, Helvetica, sans-serif; font-size:12px; background-color:#FFFFFF">

<?php
} // end printmode and not includemode
print $Status;
// if the forecast text is blank, prompt the visitor to force an update

if($UnSupported) {

  print <<< EONAG
<h1>Sorry.. this country <a href="$WU_URL">forecast</a> can not be processed by this script.</h1>
<p>Locations in the United States and territories should use the <a href="http://saratoga-weather.org/scripts-carterlake.php#advforecast">carterlake NOAA script</a>.</p>
<p>Locations in Canada should use the <a href="http://saratoga-weather.org/scripts-ECforecast.php#ECforecast">Environment Canada forecast script</a>.</p>
<p>If this was a working non-USA, non-Canada forecast, then it is likely that WeatherUnderground has changed
website design and you need an <a href="http://saratoga-weather.org/scripts-WUforecast.php#WUforecast">update to the WU-forecast.php script</a> to handle this changed design.</p>

EONAG
;
}

if (strlen($WUforecasttext[0])<2 and $PrintMode and ! $UnSupported ) {

  echo '<br/><br/>Forecast blank? <a href="' . $PHP_SELF . '?force=1">Force Update</a><br/><br/>';

} 
if ($PrintMode and ($printHeading or $printIcons)) {  ?>
  <table width="<?php print $maxWidth; ?>" style="border: none;" class="WUforecast">
  <?php echo $ddMenu ?>
    <?php if($printHeading) { ?>
    <tr align="center" style="background-color: #FFFFFF;">
      <td><b>WeatherUnderground <?php echo $WUtitle; ?>: </b><span style="color: green;">
	   <?php echo $WUforecastcity; ?></span>
      </td>
    </tr>
	<?php } // end print heading
	
	if ($printIcons) {
	?>
    <tr>
      <td align="center">
	    <table width="100%" border="0" cellpadding="0" cellspacing="0">  
	<?php
	  // see if we need to fold the icon rows due to long text length
	  $doFoldRow = false; // don't assume we have to fold the row..
	  if($foldIconRow) {
		  $iTitleLen =0;
		  $iTempLen = 0;
		  $iCondLen = 0;
		  for($i=0;$i<$doNumIcons;$i++) {
			$iTitleLen += strlen(strip_tags($WUforecasttitles[$i]));
			$iCondLen += strlen(strip_tags($WUforecastcond[$i]));
			$iTempLen += strlen(strip_tags($WUforecasttemp[$i]));  
		  }
		  print "<!-- lengths title=$iTitleLen cond=$iCondLen temps=$iTempLen -->\n";
		  $maxChars = 135;
		  if($iTitleLen >= $maxChars or 
		     $iCondLen >= $maxChars or
			 $iTempLen >= $maxChars ) {
				 print "<!-- folding icon row -->\n";
				 $doFoldRow = true;
			 } 
			 
	  }
	  $startIcon = 0;
	  $finIcon = $doNumIcons;
	  $incr = $doNumIcons;
	  if ($doFoldRow) { $wdth = $wdth*2; $incr = $numIconsInFoldedRow; }
  
	for ($k=$startIcon;$k<$doNumIcons-1;$k+=$incr) { // loop over icon rows, 5 at a time until done
	  $startIcon = $k;
	  if ($doFoldRow) { $finIcon = $startIcon+$numIconsInFoldedRow; } else { $finIcon = $doNumIcons; }
	  $finIcon = min($finIcon,$doNumIcons);
	  print "<!-- start=$startIcon fin=$finIcon num=$doNumIcons -->\n";
	  if(!$doRTL) {
        print "	      <tr valign=\"top\" align=\"center\">\n";
	  } else {
        print "	      <tr valign=\"top\" align=\"center\" style=\"direction: rtl\">\n";
	  }
	  
	  for ($i=$startIcon;$i<$finIcon;$i++) {
		$ni = $doRTL?$numIconsInFoldedRow-1-$i+$startIcon+$k:$i;  
	    print "<td style=\"width: $wdth%; text-align: center;\"><span style=\"font-size: 8pt;\">$WUforecasttitles[$ni]</span></td>\n";
		
	  }
	
print "          </tr>\n";	
	  if(!$doRTL) {
        print "	      <tr valign=\"top\" align=\"center\">\n";
	  } else {
        print "	      <tr valign=\"top\" align=\"center\" style=\"direction: rtl\">\n";
	  }
	
	  for ($i=$startIcon;$i<$finIcon;$i++) {
		$ni = $doRTL?$numIconsInFoldedRow-1-$i+$startIcon+$k:$i;  
	    print "<td style=\"width: $wdth%;\">" . img_replace($WUforecasticon[$ni],$WUforecastcond[$ni],$WUforecastpop[$ni]) . "</td>\n";
	  }
	?>
          </tr>	
	      <tr valign ="top" align="center">
	<?php
	  for ($i=$startIcon;$i<$finIcon;$i++) {
		$ni = $doRTL?$numIconsInFoldedRow-1-$i+$startIcon+$k:$i;  

	    print "<td style=\"width: $wdth%; text-align: center;\"><span style=\"font-size: 8pt;\">$WUforecastcond[$ni]</span></td>\n";
	  }
	
      print "	      </tr>\n";	
	  if(!$doRTL) {
        print "	      <tr valign=\"top\" align=\"center\">\n";
	  } else {
        print "	      <tr valign=\"top\" align=\"center\" style=\"direction: rtl\">\n";
	  }
	  
	  for ($i=$startIcon;$i<$finIcon;$i++) {
		$ni = $doRTL?$numIconsInFoldedRow-1-$i+$startIcon+$k:$i;  
	    print "<td style=\"width: $wdth%; text-align: center;\">$WUforecasttemp[$ni]</td>\n";
	  }
	  ?>
          </tr>
	<?php if(! $iconDir) { // print a PoP row since they aren't using icons 
	  if(!$doRTL) {
        print "	      <tr valign=\"top\" align=\"center\">\n";
	  } else {
        print "	      <tr valign=\"top\" align=\"center\" style=\"direction: rtl\">\n";
	  }
	
	  for ($i=$startIcon;$i<$finIcon;$i++) {
		$ni = $doRTL?$numIconsInFoldedRow-1-$i+$startIcon+$k:$i;  
	    print "<td style=\"width: $wdth%; text-align: center;\">";
	    if($WUforecastpop[$ni] > 0) {
  		  print "<span style=\"font-size: 8pt; color: #009900;\">PoP: $WUforecastpop[$ni]%</span>";
		} else {
		  print "&nbsp;";
		}
		print "</td>\n";
		
	  }
	?>
          </tr>	
	  <?php } // end if iconDir ?>
      <?php if ($doFoldRow) { 
	  if(!$doRTL) {
        print "	      <tr valign=\"top\" align=\"center\">\n";
	  } else {
        print "	      <tr valign=\"top\" align=\"center\" style=\"direction: rtl\">\n";
	  }
	  
	    for ($i=$startIcon;$i<$finIcon;$i++) {
	      print "<td style=\"width: $wdth%; text-align: center;\">&nbsp;</td>\n";
      
	    }
		print "</tr>\n";
      } // end doFoldRow ?>
  <?php } // end of foldIcon loop ?>
        </table><!-- end icon table -->
     </td>
   </tr><!-- end print icons -->
   	<?php } // end print icons ?>
</table>
  <p>&nbsp;</p>
<?php } // end print header or icons

if ($PrintMode and $printText) { ?>

<table style="border: 0" width="<?php print $maxWidth; ?>" class="WUforecast">
	<?php
	  for ($i=0;$i<count($WUforecasttitles);$i++) {
        print "<tr valign =\"top\" align=\"left\">\n";
		if(!$doRTL) { // normal Left-to-right
	      print "<td style=\"width: 20%;\"><b>$WUforecasttitles[$i]</b><br />&nbsp;<br /></td>\n";
	      print "<td style=\"width: 80%;\">$WUforecasttext[$i]</td>\n";
		} else { // print RTL format
	      print "<td style=\"width: 80%; text-align: right;\">$WUforecasttext[$i]</td>\n";
	      print "<td style=\"width: 20%; text-align: right;\"><b>$WUforecasttitles[$i]</b><br />&nbsp;<br /></td>\n";
		}
		print "</tr>\n";
	  }
	?>
   </table>
<?php } // end print text ?>
<?php if ($PrintMode) { ?>
<p>&nbsp;</p>
<p>Forecast from <a href="<?php echo htmlspecialchars($fileName); ?>">WeatherUnderground</a> 
for <?php echo $WUforecastcity; ?>.
<?php if($iconType <> '.jpg') {
	print "<br/>Animated forecast icons courtesy of <a href=\"http://www.meteotreviglio.com/\">www.meteotreviglio.com</a>.";
} 
?>
</p>
<?php
} // end printmode

 if (! $IncludeMode and $PrintMode ) { ?>
</body>
</html>
<?php 
}  

 
// Functions --------------------------------------------------------------------------------

function fetchUrlWithoutHangingWU($url,$cacheurl)
   {
   global $Status;
   // Set maximum number of seconds (can have floating-point) to wait for feed before displaying page without feed
   $numberOfSeconds=4;    

   // Suppress error reporting so Web site visitors are unaware if the feed fails
   error_reporting(0);

   // Extract resource path and domain from URL ready for fsockopen

   $url = str_replace("http://","",$url);
   $urlComponents = explode("/",$url);
   $domain = $urlComponents[0];
   $resourcePath = str_replace($domain,"",$url);
   $xml = '';

   // Establish a connection
   $socketConnection = fsockopen($domain, 80, $errno, $errstr, $numberOfSeconds);

   if (!$socketConnection)
       {

       // You may wish to remove the following debugging line on a live Web site

       $Status .= "<!-- Network error: $errstr ($errno) -->\n";
       }    // end if
   else    {
       $xml = '';
       fputs($socketConnection, "GET $resourcePath HTTP/1.1\r\nHost: $domain\r\nConnection: Close\r\nCookie: Units=metric;\r\nUser-agent: PHP,WU-forecast.php,saratoga-weather.org\r\n\r\n");
   
       // Loop until end of file
       while (!feof($socketConnection))
           {
           $xml .= fgets($socketConnection, 4096);
           }    // end while

       fclose ($socketConnection);

       }    // end else

   $xml = preg_replace('|\r\n2000\r\n|is','',$xml);  // this funky string started appearing in WU website

   return($xml);

   }    // end function

// -------------------------------------------------------------------------------------------
   
 function img_replace ( $WUimage, $WUcondtext,$WUpop) {
//
// optionally replace the WeatherUnderground icon with an NWS icon instead.
// 
 global $NWSiconlist,$iconDir,$iconType,$Status;
// $WU_URL = 'http://icons-aa.wxug.com/graphics/conds/';
// $WU_URL = 'http://icons.wunderground.com/graphics/conds/2005/';
 $WU_URL = 'http://icons-pe.wxug.com/i/c/50/';
 
 $curicon = $NWSiconlist[$WUimage][0]; // translated icon (if any)

 if (!$iconDir or !$curicon) { // no change.. use WU icon
   return("<img src=\"$WU_URL$WUimage\" width=\"50\" height=\"50\" 
  alt=\"$WUcondtext\" title=\"$WUcondtext\"/>"); 
 }
  if($iconType <> '.jpg') {
	  $curicon = preg_replace('|\.jpg|',$iconType,$curicon);
  }
  $Status .= "<!-- replace icon '$WUimage' with ";
  if ($WUpop > 0) {
	$testicon = preg_replace('|'.$iconType.'|',$WUpop.$iconType,$curicon);
	if (file_exists("$iconDir$testicon")) {
	  $newicon = $testicon;
	} else {
	  $newicon = $curicon;
	}
  } else {
	$newicon = $curicon;
  }
  $Status .= "'$newicon' pop=$WUpop -->\n";

  return("<img src=\"$iconDir$newicon\" width=\"55\" height=\"58\" 
  alt=\"$WUcondtext\" title=\"$WUcondtext\"/>"); 
 
 
 }
// End of functions --------------------------------------------------------------------------

?>